extern crate irc;
extern crate futures;

use std::io::Write;
use irc::client::prelude::*;
use futures::prelude::*;
use std::thread;

pub struct IRC {
    pub connecting: bool,
    pub connected: bool,
    pub thread: thread::JoinHandle<()>,
    pub thread_state: Result<Option<bool>, irc::error::Error>,
    pub exit_reason: String,
    pub message: String,
    pub client: Client
}

impl IRC {
    pub async fn new() -> Self {
        Self {
            connecting: false,
            connected: false,
            thread: thread::spawn(move || {}),
            thread_state: Ok(Some(false)),
            exit_reason: "havent started yet".to_owned(),
            message: "".to_owned(),
            client: Client::from_config(Config {
                nickname: Some("wowboombox".to_owned()),
                server: Some("irc.oftc.net".to_owned()),
                channels: vec!["#wownero-music".to_owned(), "#wownero".to_owned()],
                proxy_type: Some(ProxyType::Socks5),
                proxy_server: Some("127.0.0.1".to_owned()),
                proxy_port: Some(19050),
                ..Default::default()
            }).await.unwrap()
        }
    }

    pub async fn default() -> Self {
        Self::new().await
    }

    pub fn write_log(&self, b: &[u8]) -> () {
        let mut file: std::fs::File = std::fs::File::create(&crate::IRC_LOG).expect("failed to create file");
        let _ = file.write(b);
        let _ = file.flush();
        ()
    }

    pub async fn identify_server(&self, client: Client) -> Result<(), irc::error::Error> {
        return client.identify();
    }

    pub async fn listen_for_messages(&self, mut client: Client) {
        
        let mut stream = client.stream().unwrap();
        let sender = client.sender();

        while let Some(message) = stream.next().await.transpose().unwrap() {
            match message.command {
                Command::PRIVMSG(ref target, ref msg) => {
                    if msg.contains(client.current_nickname()) {
                        sender.send_privmsg(target, "yo").unwrap();
                    }
                }
                _ => (),
            }
        }
    }

    // pub async fn connect(&mut self) -> String {
    //     if ! self.connected && ! self.connecting {
    //         let config = Config {
    //             nickname: Some("wowboombox".to_owned()),
    //             server: Some("irc.oftc.net".to_owned()),
    //             channels: vec!["#wownero-music".to_owned(), "#wownero".to_owned()],
    //             proxy_type: Some(ProxyType::Socks5),
    //             proxy_server: Some("127.0.0.1".to_owned()),
    //             proxy_port: Some(19050),
    //             ..Default::default()
    //         };
    //         self.write_log("connecting".as_bytes());
    //         let client = Client::from_config(config).await.unwrap();
    //         let r = client.identify();
    //         if r.is_err() {
    //             self.thread_state = Err(r.err().unwrap());
    //             self.exit_reason = "failed to connect".to_owned();
    //             self.write_log(b"shitballs, error");
    //             return "ok there".to_owned()
    //         }
    //         self.thread_state = Ok(Some(true));
    //         self.write_log(b"fine, no error");
    //         return "nope".to_owned()
    //     } else {
    //         self.write_log(b"couldnt start, already connecting or connected");
    //         return "nope".to_owned()
    //     }
    // }

    pub async fn send_message(&self, _msg: String) -> bool {
        return true
    }

    pub fn clear_message(&mut self) {
        self.message = "".to_owned();
    }

}

// pub async fn start_irc() -> Result<(), ()> {
//     let config = Config {
//         nickname: Some("wowboombox".to_owned()),
//         server: Some("irc.oftc.net".to_owned()),
//         channels: vec!["#wownero-music".to_owned(), "#wownero".to_owned()],
//         proxy_type: Some(ProxyType::Socks5),
//         proxy_server: Some("127.0.0.1".to_owned()),
//         proxy_port: Some(19050),
//         ..Default::default()
//     };

//     let mut client = Client::from_config(config).await.unwrap();

//     match client.identify() {
//         Ok(k) => {
//             let mut stream = client.stream().unwrap();
//             let sender = client.sender();

//             while let Some(message) = stream.next().await.transpose().unwrap() {
//                 match message.command {
//                     Command::PRIVMSG(ref target, ref msg) => {
//                         if msg.contains(client.current_nickname()) {
//                             sender.send_privmsg(target, "Hi!").unwrap();
//                         }
//                     }
//                     _ => (),
//                 }
//                 let mut file: std::fs::File = std::fs::File::create(&crate::IRC_LOG).expect("failed to create file");
//                 file.write(b"somebyte");
//                 // client.send(Command::NICK("example".to_owned())).unwrap();
//                 // client.send(Command::USER("user".to_owned(), "0".to_owned(), "name".to_owned())).unwrap();

//                 // let mut buf_reader = std::io::BufReader::new(file);
//                 // let mut buf_writer = std::io::BufWriter::new(message);
//                 // let mut contents = message.to_string();
//                 // buf_reader.read_to_string(&mut contents).unwrap();

//                 // file.write_all(buf_reader);
//             }
//             Ok(())
//         },
//         Err(_e) => Err(())
//     }

//     // let log_file = std::path::Path::new(crate::IRC_LOG);
//     // if ! log_file.is_file() {
//     //     let f = std::fs::File::create(log_file);
//     //     let _ = f.unwrap().write(b"connecting");
//     // }
//     // let mut file = std::fs::OpenOptions::new()
//     //     .write(true)
//     //     .append(true)
//     //     .open(log_file)
//     //     .unwrap();

//     // let mut stream = client.stream().unwrap();
//     // let sender = client.sender();

//     // while let Some(message) = stream.next().await.transpose().unwrap() {
//     //     match message.command {
//     //         Command::PRIVMSG(ref target, ref msg) => {
//     //             if msg.contains(client.current_nickname()) {
//     //                 sender.send_privmsg(target, "Hi!").unwrap();
//     //             }
//     //         }
//     //         _ => (),
//     //     }
//     //     let mut file: std::fs::File = std::fs::File::create(&crate::IRC_LOG).expect("failed to create file");
//     //     file.write(b"somebyte");
//     //     // client.send(Command::NICK("example".to_owned())).unwrap();
//     //     // client.send(Command::USER("user".to_owned(), "0".to_owned(), "name".to_owned())).unwrap();

//     //     // let mut buf_reader = std::io::BufReader::new(file);
//     //     // let mut buf_writer = std::io::BufWriter::new(message);
//     //     // let mut contents = message.to_string();
//     //     // buf_reader.read_to_string(&mut contents).unwrap();

//     //     // file.write_all(buf_reader);
//     // }

//     // Ok(())
// }